from . import preprocessing as process
from subprocess import call
import os
from .resources.locator import locate_devconsole

def makelib(src: str, out: str) -> int:
    path = locate_devconsole()
    process.generate_header(src)
    source = src.split('.')[-2] + "_.c"
    call(f"cd \"{out}\" && \"{path}\" && cl -LD \"{source}\"", shell=True)
    return 0
def build(src: str, out: str):
    path = locate_devconsole()
    call(f"cd \"{out}\" && \"{path}\" cl \"{src}\"", shell=True)


if __name__ == "__main__":
    build("./tests/add.c", './tests/build')



import os
from subprocess import call, CalledProcessError

def makelib(src: str, out: str) -> int:
    return call(["gcc", "-static", "-shared", "-o", out, src])
def build(src: str, out: str) -> int:
    return call(["gcc", "-o", out, src])


import subprocess
import json
import os
import sys

def locate_devconsole():
    path = os.path.dirname(os.path.abspath(__file__))
    output = subprocess.check_output(f"{path}\\vswhere.exe -products * -format json", shell=True)
    vswhere_output = json.loads(output)
    path = vswhere_output[0]["installationPath"]

    # Set platform to 32-bit if 64-bit is not supported
    if sys.maxsize > 2**32:
        platform = 64
    else:
        platform = 32

    return os.path.join(path, f"VC\\Auxiliary\\Build\\vcvars{platform}.bat")


import os
from windows.builder import makelib as wlib
from linux.build import makelib as llib
import sys


def make(src: str, out: str) -> int:
    # If windows, use windows\build.py
    if os.name == "nt":
        wlib(src, out)
    # If linux, use linux\build.py 
    else:
        llib(src, out)

def main():
    args = sys.argv
    print(f"makeing {args[1]} to {args[2]}")
    make(args[1], args[2])


from dataclasses import dataclass
import re

@dataclass
class cfuncition:
    raw: str
    return_type: str
    name: str
    args: dict

def findfunctions(src: str) -> list[cfuncition]:
    """
    Finds all functions in C source code
    :param src: C source code
    :return: list of functions
    """
    functions = []
    # Open source code
    with open(src, 'r') as f:
        for line in f.readlines():
            # If a letter is followed by a ( then another letter, then it is a function
            if re.search(r'[a-zA-Z]\([a-zA-Z]', line):
                declaration = line.split('{')[0]
                # Get return type
                return_type = declaration.split(' ')[0]
                # Get function name
                name = declaration.split(' ')[1].split('(')[0]
                # Get arguments
                args_dict = {}
                args = declaration.split('(')[1].split(')')[0].split(',')
                for arg in args:
                    arg = arg.strip()
                    args_dict[arg.split(' ')[-1]] = arg.split(' ')[0]
                # Iterate every argument and get type and name
                functions.append(cfuncition(declaration, return_type, name, args_dict))
    return functions

def prepend_includes(src: str, includes: list[str]) -> int:
    """
    Prepends includes to C source code
    :param src: C source code
    :param includes: list of includes
    :return: C source code with includes prepended
    """
    # Open source code
    with open(src, 'r') as f:
        lines = f.readlines()
    # Prepend includes
    for include in includes:
        lines.insert(0, '#include "{}"\n'.format(include))
    # Write source code
    with open(src.split('.')[-2] + "_.c", 'w') as f:
        f.writelines(lines)
    return 0

def generate_header(src: str) -> int:
    """
    Generates header file from C source code functions
    :param src: C source code
    :param header: output header file
    :return: 0 if successful
    """
    header = src.split('.')[-2] + '_.h'
    # Find functions
    functions = findfunctions(src)
    # Open header file
    with open(header, 'w') as f:
        # Write header
        f.write("#define DllExport   __declspec( dllexport )" + '\n')
        # Write functions
        for function in functions:
            f.write(f"DllExport {function.raw};" + '\n')
    # include header file in source code
    prepend_includes(src, [header.split('/')[-1].split('\\')[-1]])
    return 0

if __name__ == "__main__":
    generate_header('./tests/add.c', './tests/add.h')